@@ -22,27 +22,27 @@ public:
 
     xcb_window_t trayWindow();
     void requestSystemTrayWindowDock(xcb_window_t window) const;
-    QRect systemTrayWindowGlobalGeometry(xcb_window_t window) const;
 
     void notifyManagerClientMessageEvent(const xcb_client_message_event_t *);
 
-    void handleDestroyNotifyEvent(const xcb_destroy_notify_event_t *) Q_DECL_OVERRIDE;
+    void handleDestroyNotifyEvent(const xcb_destroy_notify_event_t *) override;
 
+    xcb_visualid_t visualId();
 signals:
     void systemTrayWindowChanged(QScreen *screen);
 
 private:
     explicit QXcbSystemTrayTracker(QXcbConnection *connection,
                                    xcb_atom_t trayAtom,
-                                   xcb_atom_t selection,
-                                   QObject *parent = 0);
+                                   xcb_atom_t selection);
     static xcb_window_t locateTrayWindow(const QXcbConnection *connection, xcb_atom_t selection);
     void emitSystemTrayWindowChanged();
+    xcb_visualid_t netSystemTrayVisual();
 
     const xcb_atom_t m_selection;
     const xcb_atom_t m_trayAtom;
     QXcbConnection *m_connection;
-    xcb_window_t m_trayWindow;
+    xcb_window_t m_trayWindow = 0;
 };
 
 QT_END_NAMESPACE
