@@ -16,6 +16,8 @@
 
 #include <qpa/qplatformsessionmanager.h>
 
+#ifndef QT_NO_SESSIONMANAGER
+
 QT_BEGIN_NAMESPACE
 
 class QEventLoop;
@@ -53,4 +55,6 @@ private:
 
 QT_END_NAMESPACE
 
+#endif //QT_NO_SESSIONMANAGER
+
 #endif //QXCBSESSIONMANAGER_H
